package io.invertase.firebase.admob;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.uimanager.PixelUtil;
import com.facebook.react.uimanager.SimpleViewManager;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.annotations.ReactProp;
import com.facebook.react.uimanager.events.RCTEventEmitter;
import com.google.android.gms.ads.AdSize;
import com.google.android.gms.ads.AdView;
import com.google.android.gms.ads.LoadAdError;

import javax.annotation.Nonnull;
import java.util.Map;

public class ReactNativeFirebaseAdMobBannerAdViewManager extends SimpleViewManager<AdMobView> implements AdMobBannerListener {
	private static final String REACT_CLASS = "ReactNativeFirebaseAdMobBannerView";
	private static final String EVENT_AD_LOADED = "onAdLoaded";
	private static final String EVENT_AD_FAILED_TO_LOAD = "onAdFailedToLoad";
	private static final String EVENT_AD_OPENED = "onAdOpened";
	private static final String EVENT_AD_CLOSED = "onAdClosed";
	private static final String EVENT_AD_IMPRESSION = "onAdImpression";
	private static final String EVENT_AD_LEFT_APPLICATION = "onAdLeftApplication";

	private ThemedReactContext context;

	@Nonnull
	@Override
	public String getName() {
		return REACT_CLASS;
	}

	@Nonnull
	@Override
	public AdMobView createViewInstance(@Nonnull ThemedReactContext themedReactContext) {
		context = themedReactContext;
		AdMobView view = new AdMobView(themedReactContext);
		view.addBannerView();
		view.addListener(this);
		return view;
	}

	@Override
	public Map<String, Object> getExportedCustomDirectEventTypeConstants() {
		MapBuilder.Builder<String, Object> builder = MapBuilder.builder();
		builder.put("onNativeEvent", MapBuilder.of("registrationName", "onNativeEvent"));
		return builder.build();
	}

	@ReactProp(name = "unitId")
	public void setUnitId(AdMobView view, String value) {
		view.setUnitId(value);
	}

	@ReactProp(name = "request")
	public void setRequest(AdMobView view, ReadableMap value) {
		view.setRequest(ReactNativeFirebaseAdMobCommon.buildAdRequest(value));
	}

	@ReactProp(name = "size")
	public void setSize(AdMobView view, String value) {
		AdSize size = ReactNativeFirebaseAdMobCommon.getAdSize(value, view);

		int width;
		int height;
		WritableMap payload = Arguments.createMap();

		if (size == AdSize.SMART_BANNER) {
			width = (int) PixelUtil.toDIPFromPixel(size.getWidthInPixels(view.getContext()));
			height = (int) PixelUtil.toDIPFromPixel(size.getHeightInPixels(view.getContext()));
		} else {
			width = size.getWidth();
			height = size.getHeight();
		}

		payload.putDouble("width", width);
		payload.putDouble("height", height);

		if (size != AdSize.FLUID) {
			sendEvent(view, "onSizeChange", payload);
		}
		view.setSize(size);
	}

	@Override
	protected void onAfterUpdateTransaction(AdMobView view) {
		super.onAfterUpdateTransaction(view);
		view.makeRequest();
	}

	@Override
	public void onAdLoaded(AdMobView view) {
		int top;
		int left;
		int width;
		int height;

		AdView adView = view.getAdView();
		if (view.getSize() == AdSize.FLUID) {
			//size=FLUID not loading ad, height of child FrameLayout incorrect?
			top = 0;
			left = 0;
			width = view.getWidth();
			height = view.getHeight();
		} else {
			top = adView.getTop();
			left = adView.getLeft();
			width = adView.getAdSize().getWidthInPixels(view.getContext());
			height = adView.getAdSize().getHeightInPixels(view.getContext());
		}

		adView.measure(width, height);
		adView.layout(left, top, left + width, top + height);

		WritableMap payload = Arguments.createMap();

		if (view.getSize() != AdSize.FLUID) {
			payload.putInt("width", (int) PixelUtil.toDIPFromPixel(width) + 1);
			payload.putInt("height", (int) PixelUtil.toDIPFromPixel(height) + 1);
		} else {
			payload.putInt("width", (int) PixelUtil.toDIPFromPixel(width));
			payload.putInt("height", (int) PixelUtil.toDIPFromPixel(height));
		}

		sendEvent(view, EVENT_AD_LOADED, payload);
	}

	@Override
	public void onAdFailedToLoad(AdMobView view, LoadAdError loadAdError) {
		WritableMap payload = ReactNativeFirebaseAdMobCommon.errorCodeToMap(loadAdError.getCode());
		sendEvent(view, EVENT_AD_FAILED_TO_LOAD, payload);
	}

	@Override
	public void onAdOpened(AdMobView view) {
		sendEvent(view, EVENT_AD_OPENED, null);
	}

	@Override
	public void onAdClosed(AdMobView view) {
		sendEvent(view, EVENT_AD_CLOSED, null);
	}

	@Override
	public void onAdClicked(AdMobView view) {
		sendEvent(view, EVENT_AD_LEFT_APPLICATION, null);
	}

	@Override
	public void onAdImpression(AdMobView view) {
		sendEvent(view, EVENT_AD_IMPRESSION, null);
	}

	private void sendEvent(AdMobView view, String type, WritableMap payload) {
		WritableMap event = Arguments.createMap();
		event.putString("type", type);

		if (payload != null) {
			event.merge(payload);
		}

		context.getJSModule(RCTEventEmitter.class).receiveEvent(view.getId(), "onNativeEvent", event);
	}
}
