package io.invertase.firebase.admob;

import android.app.Activity;
import android.util.Log;
import android.util.SparseArray;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.google.android.gms.ads.AdError;
import com.google.android.gms.ads.FullScreenContentCallback;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.interstitial.InterstitialAd;
import com.google.android.gms.ads.interstitial.InterstitialAdLoadCallback;

import javax.annotation.Nullable;

import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobCommon.buildAdRequest;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobCommon.getCodeAndMessageFromAdErrorCode;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_CLICKED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_CLOSED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_ERROR;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_LOADED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_OPENED;

public class ReactNativeFirebaseAdMobInterstitialModule extends FirebaseModule {
	private static final String SERVICE = "AdMobInterstitial";
	private static SparseArray<InterstitialAd> interstitialAdArray = new SparseArray<>();

	public ReactNativeFirebaseAdMobInterstitialModule(ReactApplicationContext reactContext) {
		super(reactContext, SERVICE);
	}

	private void sendInterstitialEvent(String type, int requestId, String adUnitId, @Nullable WritableMap error) {
		sendAdEvent(
				ReactNativeFirebaseAdMobEvent.EVENT_INTERSTITIAL,
				requestId,
				type,
				adUnitId,
				error
		);
	}

	private void setFullScreenCallback(final int requestId, final String adUnitId, InterstitialAd interstitialAd){
		interstitialAd.setFullScreenContentCallback(new FullScreenContentCallback() {
			@Override
			public void onAdFailedToShowFullScreenContent(AdError adError) {
				Log.i(SERVICE, requestId + " ad fail to show full screen content");
				super.onAdFailedToShowFullScreenContent(adError);
				deleteInterstitial(requestId);
			}

			@Override
			public void onAdShowedFullScreenContent() {
				Log.i(SERVICE, requestId + " ad show full screen content");
				deleteInterstitial(requestId);
				sendInterstitialEvent(AD_OPENED, requestId, adUnitId, null);
			}

			@Override
			public void onAdDismissedFullScreenContent() {
				Log.i(SERVICE, requestId + " ad dismiss full screen content");
				//sendInterstitialEvent(AD_LEFT_APPLICATION, requestId, adUnitId, null);
				sendInterstitialEvent(AD_CLOSED, requestId, adUnitId, null);
			}

			@Override
			public void onAdImpression() {
				Log.i(SERVICE, requestId + " ad impression");
				sendInterstitialEvent(AD_CLICKED, requestId, adUnitId, null);
			}
		});
	}

	private void deleteInterstitial(int requestId){
		Log.i(SERVICE, "delete interstitial " + requestId);
		InterstitialAd ad = interstitialAdArray.get(requestId);
		ad = null;
		interstitialAdArray.delete(requestId);
	}

	@ReactMethod
	public void interstitialLoad(final int requestId, final String adUnitId, final ReadableMap adRequestOptions) {
		final Activity currentActivity = getCurrentActivity();
		if (currentActivity == null) {
			WritableMap error = Arguments.createMap();
			error.putString("code", "null-activity");
			error.putString("message", "Interstitial ad attempted to load but the current Activity was null.");
			sendInterstitialEvent(AD_ERROR, requestId, adUnitId, error);
			return;
		}
		currentActivity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				// Apply AdRequest builder
				InterstitialAd.load(currentActivity, adUnitId, buildAdRequest(adRequestOptions), new InterstitialAdLoadCallback() {
					@Override
					public void onAdLoaded(InterstitialAd interstitialAd) {
						Log.i(SERVICE, requestId + " onAdLoaded");
						interstitialAdArray.put(requestId, interstitialAd);
						setFullScreenCallback(requestId, adUnitId, interstitialAd);
						sendInterstitialEvent(AD_LOADED, requestId, adUnitId, null);
					}

					@Override
					public void onAdFailedToLoad(LoadAdError loadAdError) {
						Log.i(SERVICE, requestId + " " + loadAdError.getMessage());
						deleteInterstitial(requestId);
						WritableMap error = Arguments.createMap();
						String[] codeAndMessage = getCodeAndMessageFromAdErrorCode(loadAdError.getCode());
						error.putString("code", codeAndMessage[0]);
						error.putString("message", codeAndMessage[1]);
						sendInterstitialEvent(AD_ERROR, requestId, adUnitId, error);
					}
				});
			}
		});
	}

	@ReactMethod
	public void interstitialShow(final int requestId, final ReadableMap showOptions, final Promise promise) {
		if (getCurrentActivity() == null) {
			rejectPromiseWithCodeAndMessage(promise, "null-activity", "Interstitial ad attempted to show but the current Activity was null.");
			return;
		}
		getCurrentActivity().runOnUiThread(new Runnable() {
			@Override
			public void run() {
				InterstitialAd interstitialAd = interstitialAdArray.get(requestId);
				if (interstitialAd == null) {
					rejectPromiseWithCodeAndMessage(promise, "null-interstitialAd", "Interstitial ad attempted to show but its object was null.");
					return;
				}

				if (showOptions.hasKey("immersiveModeEnabled")) {
					interstitialAd.setImmersiveMode(showOptions.getBoolean("immersiveModeEnabled"));
				} else {
					interstitialAd.setImmersiveMode(false);
				}

				interstitialAd.show(getCurrentActivity());
				promise.resolve(null);
				//rejectPromiseWithCodeAndMessage(promise, "not-ready", "Interstitial ad attempted to show but was not ready.");
			}
		});
	}
}
