package io.invertase.firebase.admob;

import android.app.Activity;
import android.util.Log;
import android.util.SparseArray;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.Promise;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.google.android.gms.ads.AdError;
import com.google.android.gms.ads.FullScreenContentCallback;
import com.google.android.gms.ads.LoadAdError;
import com.google.android.gms.ads.OnUserEarnedRewardListener;
import com.google.android.gms.ads.interstitial.InterstitialAd;
import com.google.android.gms.ads.rewarded.RewardItem;
import com.google.android.gms.ads.rewarded.RewardedAd;
import com.google.android.gms.ads.rewarded.RewardedAdLoadCallback;
import com.google.android.gms.ads.rewarded.ServerSideVerificationOptions;

import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobCommon.buildAdRequest;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobCommon.getCodeAndMessageFromAdErrorCode;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_CLICKED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_CLOSED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_ERROR;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_OPENED;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_REWARDED_EARNED_REWARD;
import static io.invertase.firebase.admob.ReactNativeFirebaseAdMobEvent.AD_REWARDED_LOADED;

public class ReactNativeFirebaseAdMobRewardedModule extends FirebaseModule {
	private static final String SERVICE = "AdMobRewarded";
	private static SparseArray<RewardedAd> rewardedAdArray = new SparseArray<>();

	public ReactNativeFirebaseAdMobRewardedModule(ReactApplicationContext reactContext) {
		super(reactContext, SERVICE);
	}

	private void sendRewardedEvent(String type, int requestId, String adUnitId, @Nullable WritableMap error, @Nullable WritableMap data) {
		sendAdEvent(
				ReactNativeFirebaseAdMobEvent.EVENT_REWARDED,
				requestId,
				type,
				adUnitId,
				error,
				data
		);
	}

	private void setOptions(RewardedAd rewardedAd, ReadableMap adRequestOptions){
		if (adRequestOptions.hasKey("serverSideVerificationOptions")) {
			ReadableMap serverSideVerificationOptions = adRequestOptions.getMap("serverSideVerificationOptions");

			if (serverSideVerificationOptions != null) {
				ServerSideVerificationOptions.Builder options = new ServerSideVerificationOptions.Builder();

				if (serverSideVerificationOptions.hasKey("userId")) {
					options.setUserId(serverSideVerificationOptions.getString("userId"));
				}


				if (serverSideVerificationOptions.hasKey("customData")) {
					options.setCustomData(serverSideVerificationOptions.getString("customData"));
				}

				rewardedAd.setServerSideVerificationOptions(options.build());
			}
		}
		boolean immersiveModeEnabled = false;
		if (adRequestOptions.hasKey("immersiveModeEnabled")) {
			immersiveModeEnabled = adRequestOptions.getBoolean("immersiveModeEnabled");
		}
		rewardedAd.setImmersiveMode(immersiveModeEnabled);
	}

	private void setFullScreenCallback(final int requestId, final String adUnitId, RewardedAd rewardedAd){
		rewardedAd.setFullScreenContentCallback(new FullScreenContentCallback() {
			@Override
			public void onAdFailedToShowFullScreenContent(AdError adError) {
				Log.i(SERVICE, requestId + " ad fail to show full screen content");
				WritableMap error = Arguments.createMap();
				String[] codeAndMessage = getCodeAndMessageFromAdErrorCode(adError.getCode());
				error.putString("code", codeAndMessage[0]);
				error.putString("message", codeAndMessage[1]);
				deleteRewardedAd(requestId);
				sendRewardedEvent(AD_ERROR, requestId, adUnitId, error, null);
			}

			@Override
			public void onAdShowedFullScreenContent() {
				Log.i(SERVICE, requestId + " ad show full screen content");
				deleteRewardedAd(requestId);
				sendRewardedEvent(AD_OPENED, requestId, adUnitId, null, null);
			}

			@Override
			public void onAdDismissedFullScreenContent() {
				Log.i(SERVICE, requestId + " ad dismiss full screen content");
				//sendInterstitialEvent(AD_LEFT_APPLICATION, requestId, adUnitId, null);
				sendRewardedEvent(AD_CLOSED, requestId, adUnitId, null, null);
			}
		});
	}
	private void deleteRewardedAd(int requestId){
		try {
			Log.d(SERVICE, "delete rewarded ad " + requestId);
			RewardedAd ad  = rewardedAdArray.get(requestId);
			ad = null;
			rewardedAdArray.delete(requestId);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	@ReactMethod
	public void rewardedLoad(final int requestId, final String adUnitId, final ReadableMap adRequestOptions) {
		final Activity activity = getCurrentActivity();
		if (activity == null) {
			WritableMap error = Arguments.createMap();
			error.putString("code", "null-activity");
			error.putString("message", "Rewarded ad attempted to load but the current Activity was null.");
			sendRewardedEvent(AD_ERROR, requestId, adUnitId, error, null);
			return;
		}
		activity.runOnUiThread(new Runnable() {
			@Override
			public void run() {
				RewardedAd.load(activity, adUnitId, buildAdRequest(adRequestOptions), new RewardedAdLoadCallback() {
					@Override
					public void onAdLoaded(@NonNull RewardedAd rewardedAd) {
						Log.d(SERVICE, requestId + " loaded");
						setOptions(rewardedAd, adRequestOptions);
						rewardedAdArray.put(requestId, rewardedAd);
						RewardItem rewardItem = rewardedAd.getRewardItem();
						WritableMap data = Arguments.createMap();
						data.putString("type", rewardItem.getType());
						data.putInt("amount", rewardItem.getAmount());
						sendRewardedEvent(AD_REWARDED_LOADED, requestId, adUnitId, null, data);					}

					@Override
					public void onAdFailedToLoad(@NonNull LoadAdError loadAdError) {
						Log.d(SERVICE, requestId + " failed " + loadAdError.getCode());
						WritableMap error = Arguments.createMap();
						String[] codeAndMessage = getCodeAndMessageFromAdErrorCode(loadAdError.getCode());
						error.putString("code", codeAndMessage[0]);
						error.putString("message", codeAndMessage[1]);
						sendRewardedEvent(AD_ERROR, requestId, adUnitId, error, null);					}
				});
			}
		});
	}

	@ReactMethod
	public void rewardedShow(final int requestId, final String adUnitId, final ReadableMap showOptions, final Promise promise) {
		if (getCurrentActivity() == null) {
			rejectPromiseWithCodeAndMessage(promise, "null-activity", "Rewarded ad attempted to show but the current Activity was null.");
			return;
		}
		getCurrentActivity().runOnUiThread(new Runnable() {
			@Override
			public void run() {
				RewardedAd rewardedAd = rewardedAdArray.get(requestId);
				rewardedAd.show(getCurrentActivity(), new OnUserEarnedRewardListener() {
					@Override
					public void onUserEarnedReward(@NonNull RewardItem rewardItem) {
						WritableMap data = Arguments.createMap();
						data.putString("type", rewardItem.getType());
						data.putInt("amount", rewardItem.getAmount());
						sendRewardedEvent(AD_REWARDED_EARNED_REWARD, requestId, adUnitId, null, data);
					}
				});
				promise.resolve(null);
			}
		});
	}
}
