//
/**
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import <React/RCTUtils.h>

#import "RNFBAdMobInterstitialModule.h"
#import "RNFBAdMobCommon.h"
#import "RNFBAdMobSharedUtils.h"
#import "RNFBAdMobInterstitialDelegate.h"
#import "RNFBAdMobRCTEventEmitter.h"

static __strong NSMutableDictionary *interstitialMap;
static __strong NSMutableDictionary *requestIdMap;

@implementation RNFBAdMobInterstitialModule
#pragma mark -
#pragma mark Module Setup

RCT_EXPORT_MODULE();

- (id)init {
  self = [super init];
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    requestIdMap = [[NSMutableDictionary alloc] init];
    interstitialMap = [[NSMutableDictionary alloc] init];
  });
  return self;
}

- (void)dealloc {
  [self invalidate];
}

- (void)invalidate {
    [interstitialMap removeAllObjects];
    [requestIdMap removeAllObjects];
}

#pragma mark -
#pragma mark Firebase AdMob Methods

RCT_EXPORT_METHOD(interstitialLoad
  :
  (nonnull
    NSNumber *)requestId
    :(NSString *)adUnitId
    :(NSDictionary *)adRequestOptions
) {    
  GADRequest *request = [RNFBAdMobCommon buildAdRequest:adRequestOptions];
  [GADInterstitialAd loadWithAdUnitID:adUnitId
                              request:request
                    completionHandler:^(GADInterstitialAd *ad, NSError *error) {
    if (error) {
      NSLog(@"Failed to load interstitial ad with error: %@", [error localizedDescription]);
      NSDictionary *codeAndMessage = [RNFBAdMobCommon getCodeAndMessageFromAdError:error];
      [RNFBAdMobInterstitialDelegate sendInterstitialEvent:ADMOB_EVENT_ERROR requestId:requestId adUnitId:adUnitId error:codeAndMessage];
      return;
    }
    GADInterstitialAd *interstitialAd = ad;
    interstitialAd.fullScreenContentDelegate = self;
    
    interstitialMap[requestId] = interstitialAd;
    requestIdMap[ad.responseInfo.responseIdentifier] = requestId;
      
    [RNFBAdMobInterstitialDelegate sendInterstitialEvent:ADMOB_EVENT_LOADED requestId:requestId adUnitId:adUnitId error:nil];
  }];
}

RCT_EXPORT_METHOD(interstitialShow
  :
  (nonnull
    NSNumber *)requestId
    :(NSDictionary *)showOptions
    :(RCTPromiseResolveBlock) resolve
    :(RCTPromiseRejectBlock) reject
) {
  GADInterstitialAd *interstitial = interstitialMap[requestId];
  if (interstitial) {
    [interstitial presentFromRootViewController:RCTPresentedViewController()];
    resolve([NSNull null]);
  } else {
    [RNFBAdMobSharedUtils rejectPromiseWithUserInfo:reject userInfo:[@{
        @"code": @"not-ready",
        @"message": @"Interstitial ad attempted to show but was not ready.",
    } mutableCopy]];
  }
}

#pragma mark -
#pragma mark GADFullScreenContentDelegate Methods

- (void)adWillPresentFullScreenContent:(GADInterstitialAd *)ad {
    NSNumber *requestId = requestIdMap[ad.responseInfo.responseIdentifier];
    if (requestId == nil) {
        return;
    }
    
    [RNFBAdMobInterstitialDelegate sendInterstitialEvent:ADMOB_EVENT_OPENED requestId:requestId adUnitId:ad.adUnitID error:nil];
}

- (void)ad:(GADInterstitialAd *)ad didFailToPresentFullScreenContentWithError:(NSError *)error {
    NSNumber *requestId = requestIdMap[ad.responseInfo.responseIdentifier];
    if (requestId == nil) {
        return;
    }
    
    NSDictionary *codeAndMessage = [RNFBAdMobCommon getCodeAndMessageFromAdError:error];
    [RNFBAdMobInterstitialDelegate sendInterstitialEvent:ADMOB_EVENT_ERROR requestId:requestId adUnitId:ad.adUnitID error:codeAndMessage];
}

- (void)adDidDismissFullScreenContent:(GADInterstitialAd *)ad {
    NSNumber *requestId = requestIdMap[ad.responseInfo.responseIdentifier];
    if (requestId == nil) {
        return;
    }
    
    [RNFBAdMobInterstitialDelegate sendInterstitialEvent:ADMOB_EVENT_CLOSED requestId:requestId adUnitId:ad.adUnitID error:nil];
}

@end
