//
/**
 * Copyright (c) 2016-present Invertase Limited & Contributors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this library except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#import <React/RCTUtils.h>

#import "RNFBAdMobRewardedModule.h"
#import "RNFBAdMobRewardedDelegate.h"
#import "RNFBAdMobCommon.h"
#import "RNFBAdMobSharedUtils.h"

static __strong NSMutableDictionary *rewardedMap;

@implementation RNFBAdMobRewardedModule
#pragma mark -
#pragma mark Module Setup

RCT_EXPORT_MODULE();

- (id)init {
  self = [super init];
  static dispatch_once_t onceToken;
  dispatch_once(&onceToken, ^{
    rewardedMap = [[NSMutableDictionary alloc] init];
  });
  return self;
}

- (void)dealloc {
  [self invalidate];
}

- (void)invalidate {
  for (NSNumber *id in [rewardedMap allKeys]) {
    GADRewardedAd * ad = rewardedMap[id];
    [rewardedMap removeObjectForKey:id];
  }
}

#pragma mark -
#pragma mark Firebase AdMob Methods

RCT_EXPORT_METHOD(rewardedLoad
  :
  (nonnull
    NSNumber *)requestId
    :(NSString *)adUnitId
    :(NSDictionary *)adRequestOptions
) {
    GADRequest *request = [RNFBAdMobCommon buildAdRequest:adRequestOptions];

    [GADRewardedAd loadWithAdUnitID:adUnitId
                              request:request
                    completionHandler:^(GADRewardedAd *ad, NSError *error) {
        if (error != nil) {
          NSDictionary *codeAndMessage = [RNFBAdMobCommon getCodeAndMessageFromAdError:error];
          [RNFBAdMobRewardedDelegate sendRewardedEvent:ADMOB_EVENT_ERROR requestId:requestId adUnitId:adUnitId error:codeAndMessage data:nil];
        } else {
          GADRewardedAd *rewardAd = ad;
          rewardAd.fullScreenContentDelegate = [RNFBAdMobRewardedDelegate sharedInstance];
          NSDictionary *serverSideVerificationOptions = [adRequestOptions objectForKey:@"serverSideVerificationOptions"];

          if (serverSideVerificationOptions != nil) {
              GADServerSideVerificationOptions *options = [[GADServerSideVerificationOptions alloc] init];

              NSString *userId = [serverSideVerificationOptions valueForKey:@"userId"];

              if (userId != nil) {
                options.userIdentifier = userId;
              }

              NSString *customData = [serverSideVerificationOptions valueForKey:@"customData"];

              if (customData != nil) {
                options.customRewardString = customData;
              }
              
            [rewardAd setServerSideVerificationOptions:options];
          }
          rewardedMap[requestId] = rewardAd;
          NSDictionary *data = @{
              @"type": rewardAd.adReward.type,
              @"amount": rewardAd.adReward.amount,
          };
          [RNFBAdMobRewardedDelegate sendRewardedEvent:ADMOB_EVENT_REWARDED_LOADED requestId:requestId adUnitId:adUnitId error:nil data:data];
        }
    }];
}

RCT_EXPORT_METHOD(rewardedShow
  :
  (nonnull
    NSNumber *)requestId
    :(NSString *)adUnitId
    :(NSDictionary *)showOptions
    :(RCTPromiseResolveBlock) resolve
    :(RCTPromiseRejectBlock) reject
) {
  GADRewardedAd *rewarded = rewardedMap[requestId];
    if (rewarded) {
        [rewarded presentFromRootViewController:RCTPresentedViewController()
                              userDidEarnRewardHandler:^ {
            NSDictionary *data = @{
                @"type": rewarded.adReward.type,
                @"amount": rewarded.adReward.amount,
            };
            [RNFBAdMobRewardedDelegate sendRewardedEvent:ADMOB_EVENT_REWARDED_EARNED_REWARD requestId:requestId adUnitId:adUnitId error:nil data:data];
            resolve([NSNull null]);
        }];
      } else {
          [RNFBAdMobSharedUtils rejectPromiseWithUserInfo:reject userInfo:[@{
              @"code": @"not-ready",
              @"message": @"Rewarded ad attempted to show but was not ready.",
          } mutableCopy]];
      }
}

@end

